<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Client;

use Psr\Http\Message\ResponseInterface;

class Zone
{
    private $client;
    private $zone;

    public function __construct(Client $client, string $zone = 'main')
    {
        $this->client = $client;
        $this->zone = $zone;
    }

    public function setZone(string $zone = 'main'): Zone
    {
        $this->zone = $zone;

        return $this;
    }

    public function getZone(): string
    {
        return $this->zone;
    }

    public function getStatus(): array
    {
        $response = $this->client->get($this->zone . '/getStatus');
        if (200 === $response->getStatusCode()) {
            return \json_decode($response->getBody()->getContents(), true);
        }

        return [];
    }

    public function getZoneInfo(string $zone = 'main'): array
    {
        $system = new System($this->client);
        $features = $system->getFeatures();
        foreach ($features['zone'] as $key => $value) {
            if (isset($value['id']) && $zone === $value['id']) {
                return $value;
            }
        }

        return [];
    }

    public function getInputsForZone(string $zone = 'main'): array
    {
        $zoneInfo = $this->getZoneInfo($zone);

        if (false === empty($zoneInfo)) {
            return $zoneInfo['input_list'];
        }

        return [];
    }

    public function setPower(string $setting = null): int
    {
        if(null === $setting) {
            return Client::INVALID_PARAM;
        }

        $setting = strtolower($setting);

        if (false === in_array($setting, ['on', 'standby', 'off'], true)) {
            throw new \InvalidArgumentException('Accepted values for power are "on, standby, and off".');
        }

        if ('off' === $setting) {
            $setting = 'standby';
        }

        $response = $this->client->get($this->zone . '/setPower', [
          'query' => [
            'power' => $setting,
          ]
        ]);

        if ($this->checkResponse($response)) {
            $body = json_decode($response->getBody()->getContents(), true);
            return (int)$body['response_code'];
        }

        return Client::UNKNOWN_ERROR;
    }

    public function setInput(string $input): int
    {
        $inputs = $this->getInputsForZone($this->zone);

        if (array_key_exists($input, array_flip($inputs))) {
            $response = $this->client->get($this->zone . '/setInput', [
              'query' => [
                'input' => $input,
              ],
            ]);

            if ($this->checkResponse($response)) {
                $body = json_decode($response->getBody()->getContents(), true);
                return (int)$body['response_code'];
            }

            return Client::UNKNOWN_ERROR;
        }

        return Client::INVALID_RECEIVER_INPUT;
    }

    public function setMute(bool $enable = true): int
    {
        $response = $this->client->get($this->zone . '/setMute', [
          'query' => [
            'enable' => $this->client->boolToStr($enable)
          ]
        ]);

        if (200 === $response->getStatusCode()) {
            $body = json_decode($response->getBody()->getContents(), true);
            return (int)$body['response_code'];
        }

        return Client::UNKNOWN_ERROR;
    }

    public function changeVolume(int $level = 81): int
    {
        $status = $this->getStatus();

        $levels = range(1, (int)$status['max_volume']);
        $levels2 = range(80, 0, 0.5);
        $volumeMap = array_combine(array_values($levels), $levels2);
        $volumeMap = array_map(function ($value) {
            return (string)$value;
        }, $volumeMap);

        $volumeMap = array_flip($volumeMap);
        $volume = $volumeMap[(string)$level] ?? null;

        if (null === $volumeMap) {
            throw new \Exception('Volume setting doesn\'t exist');
        }

        $volume = $volume > (int)$status['max_volume'] ? (int)$status['max_volume'] : $volume;

        $response = $this->client->get($this->zone . '/setVolume', [
          'query' => [
            'volume' => $volume,
          ],
        ]);

        if ($this->checkResponse($response)) {
            $body = json_decode($response->getBody()->getContents(), true);
            return (int)$body['response_code'];
        }

        return Client::UNKNOWN_ERROR;
    }

    public function changeVolumeByPercent(float $percent): int
    {
        $status = $this->getStatus();
        $volume = ceil(($percent / 100) * $status['max_volume']);
        $response = $this->client->get($this->zone . '/setVolume', [
          'query' => [
            'volume' => $volume,
          ],
        ]);

        if ($this->checkResponse($response)) {
            $body = json_decode($response->getBody()->getContents(), true);
            return (int)$body['response_code'];
        }

        return Client::UNKNOWN_ERROR;
    }

    public function increaseVolume(float $step = 1): int
    {
        $response = $this->client->get($this->zone . '/setVolume', [
          'query' => [
            'volume' => 'up',
            'step' => $step,
          ],
        ]);

        if ($this->checkResponse($response)) {
            $body = json_decode($response->getBody()->getContents(), true);
            return (int)$body['response_code'];
        }

        return Client::UNKNOWN_ERROR;
    }

    public function decreaseVolume(float $step = 1): int
    {
        $response = $this->client->get($this->zone . '/setVolume', [
          'query' => [
            'volume' => 'down',
            'step' => $step,
          ],
        ]);

        if ($this->checkResponse($response)) {
            $body = json_decode($response->getBody()->getContents(), true);
            return (int)$body['response_code'];
        }

        return Client::UNKNOWN_ERROR;
    }

    protected function checkResponse(ResponseInterface $response): bool
    {
        return 200 === $response->getStatusCode();
    }
}
