<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Command;

use Dflydev\DotAccessData\Data;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Exception\InvalidArgumentException;
use Symfony\Component\Console\Exception\LogicException;
use Symfony\Component\Console\Exception\RuntimeException;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Sets, Deletes, and Gets configuration values.
 */
class ConfigurationCommand extends Command
{
    private $directory;

    /**
     * @var SymfonyStyle
     */
    private $io;

    /**
     * @var string
     */
    private $configFile;

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\RuntimeException
     */
    public function __construct(string $directory)
    {
        if (false === is_dir($directory)) {
            throw new RuntimeException(sprintf('%s does not exist', $directory));
        }

        $this->directory = $directory;
        $this->configFile = sprintf('%s/config/config.json', $this->directory);

        parent::__construct();
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this->setName('config')
            ->setDescription('set and get config items')
            ->addArgument('action', InputArgument::REQUIRED, 'This can either be set or get')
            ->addArgument('key', InputArgument::REQUIRED, 'Item key')
            ->addArgument('value', InputArgument::OPTIONAL, 'Value to set.');
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @throws \Symfony\Component\Console\Exception\LogicException
     */
    public function interact(InputInterface $input, OutputInterface $output): void
    {
        $this->ensureActionExists($input->getArgument('action'));
        $this->setIo($input, $output);

        if (strtolower($input->getArgument('action')) === 'set') {
            if (null === $input->getArgument('value')) {
                throw new LogicException('Can\'t set an item without a value');
            }
        }
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @throws \Symfony\Component\Console\Exception\RuntimeException
     * @throws \Symfony\Component\Filesystem\Exception\IOException
     */
    public function execute(InputInterface $input, OutputInterface $output)
    {
        $action = $input->getArgument('action');
        $key = $input->getArgument('key');
        $value = $input->getArgument('value');

        $this->createConfigIfNotExists();

        $configData = \json_decode(file_get_contents($this->configFile), true);

        if (null === $configData) {
            throw new RuntimeException('jSON : ' . json_last_error_msg(), json_last_error());
        }

        $config = new Data($configData);

        switch (strtolower($action)) {
            case 'set':
                $config->set($key, $value);
                $result = file_put_contents($this->configFile, \json_encode($config->export(), JSON_PRETTY_PRINT));
                return is_bool($result) ? (int)$result : $result;
                break;
            case 'get':
                $this->io->writeln($config->get($key));
                return 0;
                break;
            case 'del':
            case 'delete':
            case 'remove':
                $config->remove($key);
                $result = file_put_contents($this->configFile, \json_encode($config->export(), JSON_PRETTY_PRINT));
                return is_bool($result) ? (int)$result : $result;
                break;
            default:
                throw new InvalidArgumentException('Action ' . $action . ' does not exist.  Actions are set, get and del');
                break;
        }
    }

    /**
     * @param \Symfony\Component\Console\Input\InputInterface $input
     * @param \Symfony\Component\Console\Output\OutputInterface $output
     */
    public function setIo(InputInterface $input, OutputInterface $output): void
    {
        $this->io = new SymfonyStyle($input, $output);
    }

    /**
     * @throws \Symfony\Component\Console\Exception\RuntimeException
     * @throws \Symfony\Component\Filesystem\Exception\IOException
     * @return bool
     *
     */
    private function createConfigIfNotExists(): bool
    {
        $fileSystem = new Filesystem();

        if (false === $fileSystem->exists($this->configFile)) {
            $this->io->warning(sprintf('Config file %s does not exist.', $this->configFile));
            $this->io->note('Trying to create ...');
            $fileSystem->mkdir([$this->directory . '/config']);
            $fileSystem->dumpFile($this->configFile, '{}');
        }

        if (false === file_exists($this->configFile)) {
            throw new RuntimeException(
                sprintf(
                    'Could not create config file in %s/config/config.json',
                    $this->directory
                )
            );
        }

        return true;
    }

    /**
     * @param string $action
     *
     * @return bool
     */
    private function ensureActionExists(string $action): bool
    {
        $actions = [
          'set',
          'get',
          'del',
          'delete',
          'remove',
        ];

        if (array_key_exists($action, array_flip($actions))) {
            return true;
        }

        return false;
    }
}
