<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Console\Command;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception\TableExistsException;
use Doctrine\DBAL\Schema\Column;
use Doctrine\DBAL\Schema\SchemaConfig;
use Doctrine\DBAL\Schema\Table;
use Doctrine\DBAL\Types\Type;
use Ramsey\Uuid\Doctrine\UuidType;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Exception\RuntimeException;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ConfirmationQuestion;
use Symfony\Component\Console\Style\SymfonyStyle;

/**
 * Creates the database.
 */
class CreateDatabaseCommand extends Command
{
    /**
     * @var \Doctrine\DBAL\Connection
     */
    private $connection;

    /**
     * @param \Doctrine\DBAL\Connection $connection
     *
     * @throws \Symfony\Component\Console\Exception\LogicException
     */
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;

        parent::__construct($this->getName());
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this
            ->setName('db:create')
            ->setDescription('Creates the database')
            ->addOption(
                'force',
                'f',
                InputOption::VALUE_NONE,
                'Force creation if database already exists.'
            );
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\RuntimeException
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function interact(InputInterface $input, OutputInterface $output): void
    {
        if (false === extension_loaded('sqlite3')) {
            throw new RuntimeException('The sqlite3 extension is required.');
        }

        if (true === $input->getOption('force') && file_exists(realpath($this->connection->getDatabase()))) {
            $io = new SymfonyStyle($input, $output);

            $answer = $io->askQuestion(new ConfirmationQuestion(
                'Are you sure you want to remove the database?',
                false
            ));
            if (false === $answer) {
                exit(0);
            }

            $this->connection->getSchemaManager()->dropTable('users');
        }
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     *
     * @throws \Doctrine\DBAL\DBALException
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        if (true === $input->getOption('force') && true === $input->getOption('no-interaction')) {
            $this->connection->getSchemaManager()->dropTable('users');
        }

        $uniqueOption = [
          'customSchemaOptions' => [
            'unique' => true,
          ]
        ];
        $io = new SymfonyStyle($input, $output);
        $table = new Table('users');
        $table->addColumn('id', 'uuid', array_merge($uniqueOption));
        $table->addColumn('user_name', Type::STRING, array_merge($uniqueOption, ['length' => 32]));
        $table->addColumn('password', Type::STRING, array_merge($uniqueOption, ['length' => 64]));
        $table->addColumn('api_key', Type::STRING, array_merge($uniqueOption, ['length' => 128]));
        $table->addColumn('api_secret', Type::STRING, array_merge($uniqueOption, ['length' => 192]));
        $table->addColumn('api_token', Type::STRING, array_merge($uniqueOption, ['notnull' => false, 'length' => 240]));
        $table->addColumn('api_token_expire', Type::DATETIME, ['notnull' => false]);
        $table->addColumn('created_at', Type::DATETIME);
        $table->addColumn('updated_at', Type::DATETIME, ['default' => 'CURRENT_TIMESTAMP']);
        $table->addUniqueIndex(['id'], 'id_idx');
        $table->addUniqueIndex(['user_name', 'api_key', 'api_secret', 'api_token'], 'uniq_idx');
        $table->addIndex(['id', 'user_name', 'api_key', 'api_secret', 'api_token'], 'search_idx');

        try {
            $this
              ->connection
              ->getSchemaManager()
              ->createTable($table);
        } catch (TableExistsException $t) {
            $io->error([$t->getMessage(), 'Try the command again using the --force switch or remove the database.']);
            return $t->getErrorCode();
        } catch (\Exception $e) {
            $io->error($e->getMessage());
            return 1;
        }

        return 0;
    }
}
