<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Console\Command;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception\NotNullConstraintViolationException;
use Doctrine\DBAL\Exception\UniqueConstraintViolationException;
use Ramsey\Uuid\Uuid;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Exception\InvalidArgumentException;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

/**
 * Creates a user.
 */
class CreateUserCommand extends Command
{
    /**
     * @var \Doctrine\DBAL\Connection
     */
    private $connection;

    /**
     * @var string
     */
    private $userName;

    /**
     * @var string
     */
    private $password;

    /**
     * @param \Doctrine\DBAL\Connection $connection
     *
     * @throws \Symfony\Component\Console\Exception\LogicException
     */
    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
        parent::__construct();
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function configure(): void
    {
        $this
            ->setName('user:create')
            ->setDescription('Create a new user.')
            ->addOption(
                'username',
                'u',
                InputOption::VALUE_REQUIRED,
                'The username for the account'
            )
            ->addOption(
                'password',
                'p',
                InputOption::VALUE_REQUIRED,
                'The password for the account'
            )
            ;
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     */
    public function interact(InputInterface $input, OutputInterface $output): void
    {
        $io = new SymfonyStyle($input, $output);

        if (null === $input->getOption('username')) {
            $this->userName = $io->ask(
                'Username ',
                null,
                function ($value) {
                    return $this->validateUserName($value);
                }
            );
        } else {
            $this->userName = $this->validateUserName($input->getOption('username'));
        }

        if (null === $input->getOption('password')) {
            $password = $io->askHidden(
                'Password ',
                function ($value) {
                    return $this->validatePassword($value);
                }
            );
            $this->password = $this->hashPassword($password);
        } else {
            $password = $this->validatePassword($input->getOption('password'));
            $this->password = $this->hashPassword($password);
        }
    }

    /**
     * {@inheritDoc}
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @throws \Doctrine\DBAL\DBALException
     */
    public function execute(InputInterface $input, OutputInterface $output): ?int
    {
        if (true === $input->getOption('no-interaction') &&
          (null === $input->getOption('username') ||
           null === $input->getOption('password'))
        ) {
            throw new InvalidArgumentException('The options username and password are required when running in non-interactive mode.');
        }

        if (true === $input->getOption('no-interaction')) {
            $this->userName = $this->validateUserName($input->getOption('username'));
            $password = $this->validatePassword($input->getOption('password'));
            $this->password = $this->hashPassword($password);
        }

        $io = new SymfonyStyle($input, $output);
        /*
         * (user_name, password, api_key, api_secret, created_at);
         */
        $sql =
          'INSERT INTO users ' .
          '(id, user_name, password, api_key, api_secret, created_at) ' .
          'VALUES(:id, :userName, :password, :apiKey, :apiSecret, CURRENT_TIMESTAMP)';

        $stmt = $this
          ->connection
          ->prepare($sql);

        try {
            $stmt->execute([
              'id' => Uuid::uuid4(),
              'userName' => $this->userName,
              'password' => $this->password,
              'apiKey' => $this->generateKey(128),
              'apiSecret' => $this->generateKey(192),
            ]);
        } catch (UniqueConstraintViolationException $e) {
            $message = explode(PHP_EOL, $e->getMessage());
            $io->error([get_class($e), $message[count($message) - 1]]);
            return 1;
        } catch (NotNullConstraintViolationException $e) {
            $message = explode(PHP_EOL, $e->getMessage());
            $io->error([get_class($e), $message[count($message) - 1]]);
            return 1;
        } catch (\Exception $e) {
            $io->error([get_class($e), $e->getMessage()]);
            return 1;
        }

        return 0;
    }

    /**
     * Generates a key for the api_key and api_secret.
     *
     * @param int $length
     *
     * @return string
     */
    private function generateKey(int $length): string
    {
        $result = base64_encode(bin2hex(random_bytes($length*12)));
        return substr($result, 32, $length);
    }

    /**
     * @param string $value
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @return string
     *
     */
    private function validatePassword(string $value): string
    {
        if (strlen($value) < 12) {
            throw new InvalidArgumentException('Password must be at least 12 characters.');
        }

        return $value;
    }

    private function hashPassword(string $password): string
    {
        /** @noinspection PhpUndefinedConstantInspection */
        $hashingAlgorithm = true === defined('PASSWORD_ARGON2I')
          ? PASSWORD_ARGON2I
          : PASSWORD_DEFAULT;

        return password_hash($password, $hashingAlgorithm);
    }
    /**
     * @param string $value
     *
     * @throws \Symfony\Component\Console\Exception\InvalidArgumentException
     * @return string
     *
     */
    private function validateUserName(string $value): string
    {
        if (strlen($value) > 32) {
            throw new InvalidArgumentException('User name must be less than 32 characters.');
        }
        if (strlen($value) < 5) {
            throw new InvalidArgumentException('User name must be more than 4 characters.');
        }
        return $value;
    }
}
