<?php declare(strict_types=1);

/*
 *     Copyleft (C) 2017  http://sikofitt.com sikofitt@gmail.com
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Yamaha\AV\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Types\Type;
use Doctrine\ORM\EntityRepository;
use Doctrine\ORM\Mapping;

class UserRepository
{
    private $connection;

    public function __construct(Connection $connection)
    {
        $this->connection = $connection;
    }
    public function getUserById(string $id): array
    {
        $result = $this
          ->connection
          ->createQueryBuilder()
          ->select($this->getResultValues())
          ->from('users')
          ->where('id = :id')
          ->setParameter('id', $id)
          ->execute()
          ->fetch();

        return false === $result ? [] : $result;
    }

    public function getUserByUserName(string $userName): array
    {
        $result = $this
          ->connection
          ->createQueryBuilder()
          ->select($this->getResultValues())
          ->from('users')
          ->where('user_name = :user_name')
          ->setParameter('user_name', $userName)
          ->execute()->fetch();

        return false === $result ? [] : $result;
    }

    public function getUserByKeyAndSecret(string $key, string $secret): array
    {
        $result = $this
          ->connection
          ->createQueryBuilder()
          ->select($this->getResultValues())
          ->from('users')
          ->where('api_key = :api_key', 'api_secret = :api_secret')
          ->setParameter('api_key', $key)
          ->setParameter('api_secret', $secret)
          ->execute()
          ->fetch();

        return false === $result ? [] : $result;
    }

    public function getUserByToken(string $token): array
    {
        $result = $this
          ->connection
          ->createQueryBuilder()
          ->select($this->getResultValues())
          ->from('users')
          ->where('api_token = :api_token')
          ->setParameter('api_token', $token)
          ->execute()
          ->fetch();

        return false === $result ? [] : $result;
    }


    public function isTokenExpired(array $user): bool
    {

        if (false === empty($user)) {
            if (null === $user['api_token_expire'] || null === $user['api_token']) {
                return true;
            }

            $expireDate = $this->connection->convertToPHPValue($user['api_token_expire'], Type::DATETIME);
            $currentDate = new \DateTime('now');

            return $currentDate->format('U') > $expireDate->format('U');
        }

        return true;
    }

    public function getApiToken(string $userName): ?array
    {
            $statement = $this->connection->prepare('SELECT api_token, api_token_expire FROM users WHERE user_name = :user_name');
            $statement->bindValue('user_name', $userName);
            $statement->execute();
            $result = $statement->fetch();
            if(false === $result) {
                return null;
            }
            $result['api_token_expire'] = $this->connection->convertToPHPValue($result['api_token_expire'], Type::DATETIME);
            return $result;
    }

    public function createApiTokenForUser(string $userName): bool
    {
        $this->connection->beginTransaction();

        $statement = $this->connection->prepare('UPDATE users set api_token=:api_token, api_token_expire=:api_token_expire where user_name = :user_name');
        $expire = new \DateTime('now + 1 day');
        $expire = $this->connection->convertToDatabaseValue($expire, Type::DATETIME);

        try {
            $statement->execute([
                      'api_token' => $this->generateKey(240),
                      'api_token_expire' => $expire,
                      'user_name' => $userName,
                    ]);
        } catch (\Exception $e) {
            trigger_error($e->getMessage(), E_ERROR);
            return false;
        }

        $this->connection->commit();

        return (bool)$statement->rowCount();
    }

    private function getResultValues(): array
    {
        return ['id','user_name', 'password', 'api_token','api_key','api_secret','api_token_expire','created_at','updated_at'];
    }

    /**
     * Generates a key for the api_key and api_secret.
     *
     * @param int $length
     *
     * @return string
     */
    private function generateKey(int $length): string
    {
        $result = base64_encode(bin2hex(random_bytes($length*12)));
        return substr($result, 32, $length);
    }
}
